-- Deploy 0814-powerups:accomplishment_comments/comments to pg
-- requires: accomplishments/accomplishments

BEGIN;

\ir ../../defaults.sql

create table :"powerups_schema".powerup_accomplishment_comments (
  id uuid primary key default uuid_generate_v1mc(),
  sender_id uuid default :"id_schema".my_account_id(),
  powerup_accomplishment_id uuid not null,
  original_comment_id uuid,
  original_comment_deleted boolean,
  body text not null,
  sent_at timestamptz,
  created_at timestamptz not null default current_timestamp,
  updated_at timestamptz not null default current_timestamp,
  constraint sender
    foreign key (sender_id)
    references :"id_schema".users (id)
    on update cascade on delete set null,
  constraint powerup_accomplishment
    foreign key (powerup_accomplishment_id)
    references :"powerups_schema".powerup_accomplishments (id)
    on update cascade on delete cascade,
  constraint original_comment
    foreign key (original_comment_id)
    references :"powerups_schema".powerup_accomplishment_comments (id)
    on update cascade on delete set null,
  constraint updated_at_gte_created_at
    check (updated_at >= created_at)
);

comment on constraint powerup_accomplishment on :"powerups_schema".powerup_accomplishment_comments is E'@foreignFieldName comments';
comment on constraint original_comment on :"powerups_schema".powerup_accomplishment_comments is E'@foreignFieldName responses';

create index powerup_accomplishment_comments_on_sender_id on :"powerups_schema".powerup_accomplishment_comments using hash (sender_id);
create index powerup_accomplishment_comments_on_powerup_accomplishment_id on :"powerups_schema".powerup_accomplishment_comments using hash (powerup_accomplishment_id);
create index powerup_accomplishment_comments_on_original_comment_id on :"powerups_schema".powerup_accomplishment_comments using hash (original_comment_id);
create index powerup_accomplishment_comments_on_sent_at on :"powerups_schema".powerup_accomplishment_comments using brin (sent_at);
create index powerup_accomplishment_comments_on_created_at on :"powerups_schema".powerup_accomplishment_comments using brin (created_at);
create index powerup_accomplishment_comments_on_updated_at on :"powerups_schema".powerup_accomplishment_comments (updated_at);

create trigger t020_auto_set_updated_by_id
before insert or update
on :"powerups_schema".powerup_accomplishment_comments
for each row
execute function :"id_schema".my_account_id_in_column('sender_id');

create trigger t800_auto_increase_updated_at
before update
on :"powerups_schema".powerup_accomplishment_comments
for each row
execute function :"utils_schema".auto_increase_updated_at();

-- save in responses, that their original comment was deleted
create function :"powerups_schema".mark_original_comment_as_deleted()
returns trigger
as $$
begin
  new.original_comment_deleted := true;
  return new;
end;
$$ language plpgsql;

create trigger t020_mark_original_comment_as_deleted
before update of original_comment_id
on :"powerups_schema".powerup_accomplishment_comments
for each row
when (old.original_comment_id is not null and new.original_comment_id is null)
execute function :"powerups_schema".mark_original_comment_as_deleted();

COMMIT;
