-- Deploy 0814-powerups:accomplishments/accomplishments to pg
-- requires: powerups/powerups

BEGIN;

\ir ../../defaults.sql

create table :"powerups_schema".powerup_accomplishments (
  id uuid primary key default uuid_generate_v1mc(),
  actor_id uuid default :"id_schema".my_account_id(),
  powerup_id uuid not null,
  note text,
  accomplished_at timestamptz,
  created_at timestamptz not null default current_timestamp,
  updated_at timestamptz not null default current_timestamp,
  constraint actor
    foreign key (actor_id)
    references :"id_schema".users (id)
    on update cascade on delete set null,
  constraint powerup
    foreign key (powerup_id)
    references :"powerups_schema".powerups (id)
    on update cascade on delete restrict,
  constraint updated_at_gte_created_at
    check (updated_at >= created_at)
);

comment on constraint powerup on :"powerups_schema".powerup_accomplishments is E'@foreignFieldName accomplishments';

create index powerup_accomplishments_on_powerup_id on :"powerups_schema".powerup_accomplishments using hash (powerup_id);
create index powerup_accomplishments_on_actor_id on :"powerups_schema".powerup_accomplishments using hash (actor_id);
create index powerup_accomplishments_on_accomplished_at on :"powerups_schema".powerup_accomplishments using brin (accomplished_at);
create index powerup_accomplishments_on_created_at on :"powerups_schema".powerup_accomplishments using brin (created_at);
create index powerup_accomplishments_on_updated_at on :"powerups_schema".powerup_accomplishments (updated_at);
create index powerup_accomplishments_on_note_fulltext on :"powerups_schema".powerup_accomplishments using gin (to_tsvector('german', note));

create trigger t020_auto_set_updated_by_id
before insert or update
on :"powerups_schema".powerup_accomplishments
for each row
execute function :"id_schema".my_account_id_in_column('actor_id');

create trigger t800_auto_increase_updated_at
before update
on :"powerups_schema".powerup_accomplishments
for each row
execute function :"utils_schema".auto_increase_updated_at();

COMMIT;
