-- Deploy 0814-powerups:likes/fix-like-powerup-accomplishment to pg
-- requires: likes/for-accomplishments

BEGIN;

\ir ../../defaults.sql

create or replace function :"powerups_schema".like_powerup_accomplishment(
  in by_id uuid,
  in toggle boolean default false,
  out powerup_accomplishment :"powerups_schema".powerup_accomplishments,
  out "like" :"powerups_schema".powerup_accomplishment_likes,
  out added boolean,
  out is_existing boolean,
  out deleted boolean
  ) as $$
declare
  me users := my_account(fail_if_not_found => true);
begin
  added := false;
  deleted := false;

  -- fetch powerup accomplishment
  select * into powerup_accomplishment from powerup_accomplishments as ee where ee.id = by_id;
  if not found then
    raise no_data_found using message = 'powerup accomplishment not found';
  end if;

  -- fetch like
  select * into "like" from powerup_accomplishment_likes as eel where (user_id, powerup_accomplishment_id) = (me.id, by_id);
  is_existing := found;

  -- create like, if not found
  if not found then
    insert into powerup_accomplishment_likes (powerup_accomplishment_id, user_id)
    values (powerup_accomplishment.id, me.id)
    returning * into "like";
    added := true;
    is_existing := true;

  -- or drop like, if found, and if asked to toggle
  elsif toggle then
    delete from powerup_accomplishment_likes where id = "like".id returning * into "like";
    deleted := true;
    is_existing := false;
  end if;
end;
$$
language plpgsql
set search_path to :"powerups_schema", :"id_schema", public
volatile
strict;

COMMIT;
