-- Deploy 0814-powerups:likes/for-accomplishments to pg
-- requires: accomplishments/accomplishments
-- requires: 0814-tenants-and-users:users/my-account-id

BEGIN;

\ir ../../defaults.sql

create table :"powerups_schema".powerup_accomplishment_likes (
  id uuid primary key default uuid_generate_v1mc(),
  powerup_accomplishment_id uuid not null
    constraint powerup_accomplishment
    references :"powerups_schema".powerup_accomplishments (id)
    on update cascade on delete cascade,
  user_id uuid not null default :"id_schema".my_account_id()
    constraint "user"
    references :"id_schema".users (id)
    on update cascade on delete cascade,
  constraint one_like_per_user_and_powerup_accomplishment
    unique (powerup_accomplishment_id, user_id),
  liked_at timestamptz default current_timestamp
);

create index powerup_accomplishment_likes_powerup_accomplishment_id on :"powerups_schema".powerup_accomplishment_likes using hash (powerup_accomplishment_id);
create index powerup_accomplishment_likes_user_id on :"powerups_schema".powerup_accomplishment_likes using hash (user_id);
create index powerup_accomplishment_likes_liked_at on :"powerups_schema".powerup_accomplishment_likes using brin (liked_at);

comment on column :"powerups_schema".powerup_accomplishment_likes.powerup_accomplishment_id is E'@omit update';
comment on column :"powerups_schema".powerup_accomplishment_likes.user_id is E'@omit update';
comment on column :"powerups_schema".powerup_accomplishment_likes.liked_at is E'@omit insert,update';
comment on constraint powerup_accomplishment on :"powerups_schema".powerup_accomplishment_likes is E'@foreignFieldName likes';

alter table :"powerups_schema".powerup_accomplishment_likes
  enable row level security;

create policy manage_own_likes
on :"powerups_schema".powerup_accomplishment_likes
for all
using ("user_id" = :"id_schema".my_account_id())
with check ("user_id" = :"id_schema".my_account_id());

select format(
  $block$
    create function %1$I.powerup_accomplishments_number_of_likes(
      powerup %1$I.powerup_accomplishments
    ) returns bigint
    as $$
      select count(*)
      from %1$I.powerup_accomplishment_likes
      where powerup_accomplishment_id = powerup.id
    $$ language sql strict stable security definer;

    create function %1$I.powerup_accomplishments_my_like(
      powerup %1$I.powerup_accomplishments
    ) returns %1$I.powerup_accomplishment_likes
    as $$
      select *
      from %1$I.powerup_accomplishment_likes
      where
        (powerup_accomplishment_id, user_id) = (powerup.id, %2$I.my_account_id())
    $$ language sql strict stable;
  $block$,
  :'powerups_schema',
  :'id_schema'
)\gexec

comment on function :"powerups_schema".powerup_accomplishments_number_of_likes(:"powerups_schema".powerup_accomplishments) is E'@filterable\n@sortable';

create function :"powerups_schema".like_powerup_accomplishment(
  in by_id uuid,
  in toggle boolean default false,
  out powerup_accomplishment :"powerups_schema".powerup_accomplishments,
  out "like" :"powerups_schema".powerup_accomplishment_likes,
  out added boolean,
  out is_existing boolean,
  out deleted boolean
) as $$
declare
  me users := my_account(fail_if_not_found => true);
begin
  added := false;
  deleted := false;

  -- fetch powerup accomplishment
  select * into powerup_accomplishment from powerup_accomplishments as ee where ee.id = by_id;
  if not found then
    raise no_data_found using message = 'powerup accomplishment not found';
  end if;

  -- fetch like
  select * into "like" from powerup_accomplishment_likes as eel where user_id = me.id;
  is_existing := found;

  -- create like, if not found
  if not found then
    insert into powerup_accomplishment_likes (powerup_accomplishment_id, user_id)
    values (powerup_accomplishment.id, me.id)
    returning * into "like";
    added := true;
    is_existing := true;

  -- or drop like, if found, and if asked to toggle
  elsif toggle then
    delete from powerup_accomplishment_likes where id = "like".id returning * into "like";
    deleted := true;
    is_existing := false;
  end if;
end;
$$
language plpgsql
set search_path to :"powerups_schema", :"id_schema", public
volatile
strict;

create function :"powerups_schema".unlike_powerup_accomplishment(
  in by_id uuid,
  out "like" :"powerups_schema".powerup_accomplishment_likes
) as $$
declare
  me users := my_account(fail_if_not_found => true);
begin
  delete from powerup_accomplishment_likes
  where (powerup_accomplishment_id, "user_id") = (by_id, me.id)
  returning * into "like";
end;
$$
language plpgsql
set search_path to :"powerups_schema", :"id_schema", public
volatile
strict;

COMMIT;
