-- Deploy 0814-powerups:likes/for-powerup-accomplishment-comments to pg
-- requires: 0814-tenants-and-users:users/my-account-id
-- requires: accomplishment-comments/comments

BEGIN;

\ir ../../defaults.sql

create table :"powerups_schema".powerup_accomplishment_comment_likes (
  id uuid primary key default uuid_generate_v1mc(),
  powerup_accomplishment_comment_id uuid not null
    constraint powerup_accomplishment_comment
    references :"powerups_schema".powerup_accomplishment_comments (id)
    on update cascade on delete cascade,
  user_id uuid not null default :"id_schema".my_account_id()
    constraint "user"
    references :"id_schema".users (id)
    on update cascade on delete cascade,
  constraint one_like_per_user_and_powerup_accomplishment_comment
    unique (powerup_accomplishment_comment_id, user_id),
  liked_at timestamptz default current_timestamp
);

create index powerup_accomplishment_comment_likes_on___comment_id on :"powerups_schema".powerup_accomplishment_comment_likes using hash (powerup_accomplishment_comment_id);
create index powerup_accomplishment_comment_likes_on_user_id on :"powerups_schema".powerup_accomplishment_comment_likes using hash (user_id);
create index powerup_accomplishment_comment_likes_on_liked_at on :"powerups_schema".powerup_accomplishment_comment_likes using brin (liked_at);

comment on column :"powerups_schema".powerup_accomplishment_comment_likes.powerup_accomplishment_comment_id is E'@omit update';
comment on column :"powerups_schema".powerup_accomplishment_comment_likes.user_id is E'@omit update';
comment on column :"powerups_schema".powerup_accomplishment_comment_likes.liked_at is E'@omit insert,update';
comment on constraint powerup_accomplishment_comment on :"powerups_schema".powerup_accomplishment_comment_likes is E'@foreignFieldName likes';

alter table :"powerups_schema".powerup_accomplishment_comment_likes
  enable row level security;

create policy manage_own_likes
on :"powerups_schema".powerup_accomplishment_comment_likes
for all
using ("user_id" = :"id_schema".my_account_id())
with check ("user_id" = :"id_schema".my_account_id());

select format(
  $block$
    create function %1$I.powerup_accomplishment_comments_number_of_likes(
      comment %1$I.powerup_accomplishment_comments
    ) returns bigint
    as $$
      select count(*)
      from %1$I.powerup_accomplishment_comment_likes
      where powerup_accomplishment_comment_id = comment.id
    $$ language sql strict stable security definer;

    create function %1$I.powerup_accomplishment_comments_my_like(
      comment %1$I.powerup_accomplishment_comments
    ) returns %1$I.powerup_accomplishment_comment_likes
    as $$
      select *
      from %1$I.powerup_accomplishment_comment_likes
      where
        (powerup_accomplishment_comment_id, user_id) = (comment.id, %2$I.my_account_id())
    $$ language sql strict stable;
  $block$,
  :'powerups_schema',
  :'id_schema'
)\gexec

comment on function :"powerups_schema".powerup_accomplishment_comments_number_of_likes(:"powerups_schema".powerup_accomplishment_comments) is E'@filterable\n@sortable';

create function :"powerups_schema".like_powerup_accomplishment_comment(
  in by_id uuid,
  in toggle boolean default false,
  out powerup_accomplishment_comment :"powerups_schema".powerup_accomplishment_comments,
  out "like" :"powerups_schema".powerup_accomplishment_comment_likes,
  out added boolean,
  out is_existing boolean,
  out deleted boolean
) as $$
declare
  me users := my_account(fail_if_not_found => true);
begin
  added := false;
  deleted := false;

  -- fetch comment
  select * into powerup_accomplishment_comment from powerup_accomplishment_comments as ee where ee.id = by_id;
  if not found then
    raise no_data_found using message = 'comment not found';
  end if;

  -- fetch like
  select * into "like" from powerup_accomplishment_comment_likes as eel where (user_id, powerup_accomplishment_comment_id) = (me.id, by_id);
  is_existing := found;

  -- create like, if not found
  if not found then
    insert into powerup_accomplishment_comment_likes (powerup_accomplishment_comment_id, user_id)
    values (powerup_accomplishment_comment.id, me.id)
    returning * into "like";
    added := true;
    is_existing := true;

  -- or drop like, if found, and if asked to toggle
  elsif toggle then
    delete from powerup_accomplishment_comment_likes where id = "like".id returning * into "like";
    deleted := true;
    is_existing := false;
  end if;
end;
$$
language plpgsql
set search_path to :"powerups_schema", :"id_schema", public
volatile
strict;

create function :"powerups_schema".unlike_powerup_accomplishment_comment(
  in by_id uuid,
  out "like" :"powerups_schema".powerup_accomplishment_comment_likes
) as $$
declare
  me users := my_account(fail_if_not_found => true);
begin
  delete from powerup_accomplishment_comment_likes
  where (powerup_accomplishment_comment_id, "user_id") = (by_id, me.id)
  returning * into "like";
end;
$$
language plpgsql
set search_path to :"powerups_schema", :"id_schema", public
volatile
strict;

COMMIT;
