-- Deploy 0814-powerups:powerups/powerups to pg
-- requires: types/powerup-visibility
-- requires: 0814-tenants-and-users:users/my-account-id-in-column
-- requires: 0814-tenants-and-users:tenants/my-tenant-id-in-column
-- requires: 0814-utils:increase-updated-at

BEGIN;

\ir ../../defaults.sql

create extension if not exists pg_trgm;

create table :"powerups_schema".powerups (
  id uuid primary key default uuid_generate_v1mc(),
  tenant_id uuid not null default coalesce(
    :"id_schema".my_tenant_id(),
    '00000000-0000-0000-0000-000000000000'
  ),
  "name" varchar not null,
  "description" text,
  is_visible_to :"powerups_schema".powerup_visibility not null default 'me',
  is_open_for_new_members boolean not null default false,
  created_at timestamptz not null default current_timestamp,
  updated_at timestamptz not null default current_timestamp,
  updated_by_id uuid default :"id_schema".my_account_id(),
  constraint tenant
    foreign key (tenant_id)
    references :"id_schema".tenants (id)
    on update cascade on delete cascade,
  constraint "updated_by"
    foreign key (updated_by_id)
    references :"id_schema".users (id)
    on update cascade on delete set null,
  constraint updated_at_gte_created_at
    check (updated_at >= created_at)
);

create index powerups_on_tenant_id on :"powerups_schema".powerups using hash (tenant_id);
create index powerups_on_updated_by_id on :"powerups_schema".powerups using hash (updated_by_id);
create index powerups_on_name on :"powerups_schema".powerups (name);
create index powerups_on_fuzzy_name on :"powerups_schema".powerups using gist (name gist_trgm_ops);
create index powerups_on_created_at on :"powerups_schema".powerups using brin (created_at);
create index powerups_on_updated_at on :"powerups_schema".powerups (updated_at);

create trigger t800_auto_increase_updated_at
before update
on :"powerups_schema".powerups
for each row
execute function :"utils_schema".auto_increase_updated_at();

create trigger t020_auto_set_updated_by_id
before insert or update
on :"powerups_schema".powerups
for each row
execute function :"id_schema".my_account_id_in_column('updated_by_id');

create trigger t020_auto_set_tenant_id
before insert
on :"powerups_schema".powerups
for each row
execute function :"id_schema".my_tenant_id_in_column('tenant_id');

COMMIT;
