-- Deploy 0814-powerups:subscriptions/policies/regulate_new_maintainers to pg
-- requires: subscriptions/my-subscriptions

BEGIN;

\ir ../../../defaults.sql

create or replace function :"powerups_schema".constrain_maintainer_changes()
returns trigger
as $$
declare
  new_is_maintainer boolean := new is not null and 'maintainer' = any(new.roles);
  old_is_maintainer boolean := old is not null and 'maintainer' = any(old.roles);
  i_am_maintainer boolean := exists (
    select from my_powerup_subscriptions()
    where powerup_id = new.powerup_id and 'maintainer' = any(roles)
  );
begin
  if new_is_maintainer and not old_is_maintainer then
    -- everyone may become maintainer, if the powerup has no maintainer
    if not exists (
      select from powerup_subscriptions
      where
        powerup_id = new.powerup_id
        and 'maintainer' = any(roles)
        and id != new.id
    ) then
      return new;
    end if;

    -- maintainers may promote other members
    if i_am_maintainer then
      return new;
    end if;

    raise exception 'you are not allowed to add maintainers to this powerup';
  end if;

  if old_is_maintainer and (not new_is_maintainer or tg_op = 'DELETE') then
    -- maintainers may delete or downgrade themselves or other maintainers
    if i_am_maintainer then
      return new;
    end if;

    raise exception 'you are not allowed to remove maintainers from this powerup';
  end if;

  return new;
end;
$$
language plpgsql
set search_path to :"powerups_schema", public
security definer;

select format(
  $block$
    create constraint trigger t800_constrain_maintainer_changes
    after insert or update of roles or delete
    on %1$I.powerup_subscriptions
    for each row
    when (
      not %2$I.owns_table(%3$L, current_user)
      and not %2$I.can_bypass_rls(current_user)
    )
    execute function %1$I.constrain_maintainer_changes();
  $block$,
  :'powerups_schema',
  :'utils_schema',
  :'powerups_schema' || '.powerup_subscriptions'
)\gexec


COMMIT;
