-- Deploy 0814-powerups:subscriptions/subscriptions to pg
-- requires: powerups/powerups
-- requires: types/powerup-role
-- requires: 0814-tenants-and-users:users/my-account-id
-- requires: 0814-utils:increase-updated-at
-- requires: 0814-utils:array-sort-distinct-in-column

BEGIN;

\ir ../../defaults.sql

create table :"powerups_schema".powerup_subscriptions (
  id uuid primary key default uuid_generate_v1mc(),
  "user_id" uuid not null default :"id_schema".my_account_id(),
  powerup_id uuid not null,
  roles :"powerups_schema".powerup_role[] not null default '{member}',
  created_at timestamptz not null default current_timestamp,
  updated_at timestamptz not null default current_timestamp,
  constraint "user"
    foreign key ("user_id")
    references :"id_schema".users (id)
    on update cascade on delete cascade,
  constraint powerup
    foreign key (powerup_id)
    references :"powerups_schema".powerups (id)
    on update cascade on delete cascade,
  constraint one_subscription_per_user_and_powerup
    unique (user_id, powerup_id),
  constraint updated_at_gte_created_at
    check (updated_at >= created_at)
);

comment on constraint powerup on :"powerups_schema".powerup_subscriptions is E'@foreignFieldName subscriptions';

create index powerup_subscriptions_on_user_id on :"powerups_schema".powerup_subscriptions using hash ("user_id");
create index powerup_subscriptions_on_powerup_id on :"powerups_schema".powerup_subscriptions using hash (powerup_id);
create index powerup_subscriptions_on_created_at on :"powerups_schema".powerup_subscriptions using brin (created_at);
create index powerup_subscriptions_on_updated_at on :"powerups_schema".powerup_subscriptions (updated_at);
create index powerup_subscriptions_on_roles on :"powerups_schema".powerup_subscriptions using gin (roles);

create trigger t800_auto_increase_updated_at
before update
on :"powerups_schema".powerup_subscriptions
for each row
execute function :"utils_schema".auto_increase_updated_at();

create trigger t020_sort_distinct_roles
before insert or update of roles
on :"powerups_schema".powerup_subscriptions
for each row
execute function :"utils_schema".array_sort_distinct_in_column('roles');

create trigger t100_auto_set_updated_by_id
before insert or update
on :"powerups_schema".powerup_subscriptions
for each row
execute function :"id_schema".my_account_id_in_column('updated_by_id');

COMMIT;
